#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Crée le Mon Feb 14 10:34:29 2022
@auteur: david ALBERTO      (www.astrolabe-science.fr)
Ce code utilise le module mendeleev (données sur les éléments chimiques)
Dans ce module, un élément est appelé par element(N) pour récupérer
les données suiantes :
    - symbole
    - group_id : c'est le numéro de colonne
    - period : c'est le numéro de ligne
    - atomic_weight : la masse atomique (en u), ici assimilée à la masse molaire atomique (en g/mol)
"""
# ---------------------------------
# Appel des modules
import matplotlib.pyplot as plt
import pandas as pd
from mendeleev import element
# from mendeleev.fetch import fetch_table

# ---------------------------------

 # Paramètres personnalisables-----
blocs=False#True : cases colorées selon les blocs. False : incolores
fig=plt.figure(figsize=(29.7/2.54,21/2.54),tight_layout=True,facecolor='cornflowerblue')

"""
Choix de police : pour la configuration, préférer une police
compacte (comme Roboto Condensed), pour gain de place.
Dans le code ci-dessous, la police LaTeX est utilisée pour
le symbole, le nom, le numéro atomique et le titre, avec l'option : 'usetex=True'.
Pour la configuration, cette option est rendue impossible
à cause de l'affichage en plusieurs lignes (symbole '\n' de retour ligne
non interprété).
"""
plt.rcParams["font.family"] = "Roboto Condensed"# ou autre police installée


# plt.rcParams.update({"text.usetex": True}) #utiliser la police LaTeX pour tout le doc (optionnel)

"""
Les paramètres suivants sont liés entre eux, pour le rendu final.
Ils doivent être ajustés manuellement :
    - dimensions de la figure
    - valeurs des paramètres d'échelle a et b
    - taille des caractères
    - les différents offsets en x et en y
"""
#facteurs d'échelle :
a=0.22 #en abscisses
b=0.28 #en ordonnées. NB : les ordonnées sont négatives

# Réglages d'écriture du symbole des éléments :
taille_symbole=16
c_symbole='k'#couleur symbole
x_offset_symbole=0.01
y_offset_symbole=0.04

# Réglages d'écriture du numéro atomique :
x_offset_Z=-0.32
y_offset_Z=-0.07
taille_Z=8
c_Z='k'#couleur numéro atomique

# Réglages d'écriture des terres rares :
y_offset_La=2.5#décalage vertical lanthanides
y_offset_Ac=y_offset_La#décalage vertical actinides

# Réglages d'écriture du nom des éléments :
x_offset_nom=-0.0
y_offset_nom=0.35
c_nom='darkblue'
taille_nom=7

# Réglages d'écriture de la config. électronique :
x_offset_conf=0.
y_offset_conf=-0.42
c_conf='darkmagenta'
taille_conf=6

# Couleurs des blocs :
c_bloc_s='orange'
c_bloc_p='lightsteelblue'
c_bloc_d='lightyellow'
c_bloc_f='yellowgreen'

#---------------------------------------------------------
def config(Z):
    """
    renvoie la configuration électronique de l'élément.
    """
    chaine=element(Z).econf
    decoupe=chaine.split()
    morceaux=len(decoupe)
    if morceaux <= 2:
        return chaine
    elif morceaux ==3 :
        ch1, ch2, ch3=chaine.split()
        return ch1+' '+ch2+'\n'+ch3
    elif morceaux == 4:
        ch1, ch2, ch3, ch4=chaine.split()
        return ch1+' '+ch2+'\n'+ch3+' '+ch4
    elif morceaux == 5:
        ch1, ch2, ch3, ch4, ch5=chaine.split()
        return ch1+' '+ch2+'\n'+ch3+' '+ch4+'\n'+ch5
#---------------------------------------------------------

#---------------------------------------------------------
noms=['Hydrogène','Hélium','Lithium','Béryllium','Bore','Carbone','Azote','Oxygène',
      'Fluor','Néon','Sodium','Magnésium','Aluminium','Silicium','Phosphore','Soufre',
      'Chlore','Argon','Potassium','Calcium','Scandium','Titane','Vanadium','Chrome',
      'Manganèse','Fer','Cobalt','Nickel','Cuivre','Zinc','Gallium','Germanium',
      'Arsenic','Sélénium','Brome','Krypton','Rubidium','Strontium','Yttrium',
      'Zirconium','Niobium','Molybdène','Technétium','Ruthénium','Rhodium',
      'Palladium','Argent','Cadmium','Indium',
'Étain','Antimoine','Tellure','Iode','Xénon','Césium','Baryum','Lanthane','Cérium',
'Praséodyme','Néodyme','Prométhium','Samarium','Europium','Gadolinium','Terbium',
'Dysprosium','Holmium','Erbium','Thulium','Ytterbium','Lutetium','Hafnium','Tantale',
'Tungstène','Rhénium','Osmium','Iridium','Platine','Or','Mercure','Thallium','Plomb',
'Bismuth','Polonium','Astate','Radon','Francium','Radium','Actinium','Thorium','Protactinium',
'Uranium','Neptunium','Plutonium','Americium','Curium','Berkelium','Californium','Einsteinium',
'Fermium','Mendelevium','Nobelium','Lawrencium','Rutherfordium','Dubnium','Seaborgium','Bohrium',
'Hassium','Meitnerium','Darmstadtium','Roentgenium','Copernicium','Nihonium','Flerovium',
'Moscovium','Livermorium','Tennessine','Oganesson',
]#le module mendeleev n'a que les noms des éléments en anglais
# ---------------------------------


#création des listes de coordonnées des éléments
"""
La ligne est la valeur de la période de l'élément.
La colonne est la valeur du paramètre "group_id" de l'élément.
Pour les terres rares, le module ne fournit pas de valeur pour "group_id" ; 
je me base donc sur la valeur de N pour établir un numéro de colonne.
Les listes "ligne" et "colonne" serviront ensuite pour toutes les indications
(masse molaire, N° atomique, symbole, nom).
Cette opération peut être faite une seule fois, à la 
1e exécution : les résultats sont stockés dans un fichier
csv pour être réutilisés lors des exécutions ultérieures,
pour gagner du temps.
"""
# ligne=[]
# colonne=[]

# for N in range(1,118+1):
#     if N>=58 and N<=71:#lanthanides
#         colonne.append((3+N-57)*a)
#         ligne.append((element(N).period+y_offset_La)*b)
#     elif N>=90 and N<=103:#actinides
#         colonne.append((3+N-89)*a)
#         ligne.append((element(N).period+y_offset_Ac)*b)
#     else:
#         colonne.append((element(N).group_id)*a)
#         ligne.append((element(N).period)*b)

# #enregistrement pour usage ultérieur, afin de gagner du temps
# positions=pd.DataFrame({'ligne':ligne,'colonne':colonne})
# positions.to_csv('positions.csv')
# fin des commandes à exécuter uniquement lors de la 1e 
# compilation.
# ---------------------------------
# paramètres du graphique
ax=plt.subplot(111)

plt.title(r"$\textbf{\textsf{Tableau périodique des éléments chimiques}}$",
          fontsize=20,y=0.92,usetex=True)
plt.xlim(0,19*a)
plt.ylim(-10.5*b,0)
plt.xticks([])
plt.yticks([])
ax.set_aspect('equal')
# ---------------------------------
# À exécuter lors des compilations ultérieures, si 
# les positions sont déjà stockées dans le fichier csv :
positions=pd.read_csv("positions.csv")
colonne=positions['colonne']
ligne=positions['ligne']


# Remplissage du tableau :
for N in range(1,118+1):
    elem=element(N)
    x=colonne[N-1]
    y=ligne[N-1]
    #symbole :
    plt.text(x+x_offset_symbole,-y+y_offset_symbole,elem.symbol,fontsize=taille_symbole,
             va='center',ha='center',usetex=True)
    if blocs==True:
        if elem.block=='s':
            couleur_case=c_bloc_s
        elif elem.block=='p':
            couleur_case=c_bloc_p
        elif elem.block=='d':
            couleur_case=c_bloc_d
        else : couleur_case=c_bloc_f
    else: couleur_case='none'
    rectangle=plt.Rectangle((x-0.5*a,-(y+0.5*b)),a,b,facecolor=couleur_case,edgecolor='k')#case
    ax.add_patch(rectangle)
    if N in [91,101,104,110,116]:#éléments aux noms longs
        tailleDuNom=6
    else:tailleDuNom=taille_nom
    # numéro atomique :
    plt.text(x+x_offset_Z*a,-y+y_offset_Z*b,N,c=c_Z,
             fontsize=taille_Z,ha='center',usetex=True)
    #nom :
    plt.text(x+x_offset_nom*a,-y+y_offset_nom*b,r'\textsf{%s}'%(noms[N-1]),c=c_nom,fontsize=tailleDuNom,
             ha='center',usetex=True)
    # plt.text(x+x_offset_nom*a,-y+y_offset_nom*b,noms[N-1],c=c_nom,
             # fontsize=tailleDuNom,ha='center')
    tailleDuNom=taille_nom
    #configuration :
    plt.text(x+x_offset_conf*a,-y+y_offset_conf*b,config(N),c=c_conf,fontsize=taille_conf,ha='center')



# Trait pour les terres rares :
plt.plot([3.5*a,3.5*a],[-7.5*b,-8.0*b],'--',c='gray',lw=1)

ax=plt.gca()#nommage du graphique courant
ax.text(1.01,0,'D. Alberto (www.astrolabe-science.fr)',
        rotation=90,transform=ax.transAxes,fontsize=7)

NomFichier="TableauPeriodique_config"
fig.savefig(NomFichier+'.png',dpi=400)
fig.savefig(NomFichier+'.pdf')